#!/bin/sh
#------------------------------------------------------------------------------
# /var/install/config.d/lprng_power-update.sh - parameter update script
#
# Copyright (c) 2010-2018  The Eisfair Team, team(at)eisfair(dot)org
#
# Creation:     2010-07-08 jed
# Last Update:  $Id: lprng_power-update.sh 55729 2018-06-12 08:28:37Z jed $
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#------------------------------------------------------------------------------

# include eislib
. /var/install/include/eislib
. /var/install/include/configlib
. /var/install/include/jedlib

# debug mode: true/false
#debug=true
if ${debug:-false}
then
    exec 2>/tmp/$(basename ${0})-trace$$.log
    set -x
    ask_debug=true
    export ask_debug
fi

#------------------------------------------------------------------------------
# function: variable is set
#
# input:  $1 - variable name
# return:  0 - variable set
#          1 - variable not set
#------------------------------------------------------------------------------
variable_set ()
{
    local _vs_var1
    local _vs_var2
    local _vs_ret

    eval _vs_var1=\$"${1}"
    eval _vs_var2=\$"{${1}+EMPTY}"

    if [ -z "${_vs_var1}" ]
    then
        if [ "${_vs_var2}" = 'EMPTY' ]
        then
            # variable is empty, but set
            _vs_ret=0
        else
            # variable is not set
            _vs_ret=1
        fi
    else
        # variable has a value
        _vs_ret=0
    fi

    return ${_vs_ret}
}

#------------------------------------------------------------------------------
# print variable only if it has been set
# $1 - variable name
# $2 - comment
#------------------------------------------------------------------------------
printsetvar ()
{
    if variable_set "${1}"
    then
        printvar "${1}" "${2}"
    fi
}

#------------------------------------------------------------------------------
# rename variables
#------------------------------------------------------------------------------
rename_variables ()
{
    local _rv_renamed
    local _rv_output_file

    _rv_renamed=0
    _rv_output_file=`mktemp -t ${module_name}.XXXXXXXXX`

    if [ ${_rv_renamed} -eq 1 ]
    then
        modflag=1
        myecho --pack ${module_name} --force-output "renaming parameter(s) ..."
        cat ${_rv_output_file}
        myecho --pack ${module_name} --force-output --info "... read documentation for renamed parameter(s)!"
    fi

    rm -f ${_rv_output_file}
}

#------------------------------------------------------------------------------
# modify variables
#------------------------------------------------------------------------------
modify_variables ()
{
    local _mv_modified
    local _mv_output_file

    _mv_modified=0
    _mv_output_file=`mktemp -t ${module_name}.XXXXXXXXX`

    if [ ${_mv_modified} -eq 1 ]
    then
        modflag=1
        myecho --pack ${module_name} --force-output "modifying parameter(s) ..."
        cat ${_mv_output_file}
        myecho --pack ${module_name} --force-output --info "... read documentation for modified parameter(s)!"
    fi

    rm -f ${_mv_output_file}
}

#------------------------------------------------------------------------------
# add variables
#------------------------------------------------------------------------------
add_variables ()
{
    local _av_added
    local _av_output_file
    local _av_idx
    local _av_imax

    _av_added=0
    _av_output_file=`mktemp -t ${module_name}.XXXXXXXXX`

    if [ ${LPRNG_POWER_N} -eq 0 ]
    then
        _av_imax=1
    else
        _av_imax=${LPRNG_POWER_N}
    fi

    _av_idx=1
    while [ ${_av_idx} -le ${_av_imax} ]
    do
        if [ -z "`grep ^LPRNG_POWER_${_av_idx}_URL_ON ${source_conf}`" ]
        then
            echo "- LPRNG_POWER_${_av_idx}_URL_ON='/r?b=1&r=0&s=1'" >> ${_av_output_file}
            export LPRNG_POWER_${_av_idx}_URL_ON='/r?b=1&r=0&s=1'
            _av_added=1
        fi

        if [ -z "`grep ^LPRNG_POWER_${_av_idx}_URL_OFF ${source_conf}`" ]
        then
            echo "- LPRNG_POWER_${_av_idx}_URL_OFF='/r?b=1&r=0&s=0'" >> ${_av_output_file}
            export LPRNG_POWER_${_av_idx}_URL_OFF='/r?b=1&r=0&s=0'
            _av_added=1
        fi

        if [ -z "`grep ^LPRNG_POWER_${_av_idx}_URL_STATUS ${source_conf}`" ]
        then
            echo "- LPRNG_POWER_${_av_idx}_URL_STATUS='/xml'" >> ${_av_output_file}
            export LPRNG_POWER_${_av_idx}_URL_STATUS='/xml'
            _av_added=1
        fi

        if [ -z "`grep ^LPRNG_POWER_${_av_idx}_URL_STATUS_FILTER ${source_conf}`" ]
        then
            echo "- LPRNG_POWER_${_av_idx}_URL_STATUS_FILTER='sed -e \"/<nv0>/!d\" -e \"s/^.*<nv0>//\" -e \"s#</nv0>.*\$##\"\'" >> ${_av_output_file}
            export LPRNG_POWER_${_av_idx}_URL_STATUS_FILTER='sed -e "/<nv0>/!d" -e "s/^.*<nv0>//" -e "s/<\/nv0>.*$//"'
            _av_added=1
        fi

        _av_idx=`expr ${_av_idx} + 1`
    done

    if [ ${_av_added} -eq 1 ]
    then
        modflag=1
        myecho --pack ${module_name} --force-output "adding new parameter(s) ..."
        cat ${_av_output_file}
        myecho --pack ${module_name} --force-output --info "... read documentation for added parameter(s)!"
    fi

    rm -f ${_av_output_file}
}

#------------------------------------------------------------------------------
# delete variables
#------------------------------------------------------------------------------
delete_variables ()
{
    local _dv_deleted
    local _dv_output_file

    _dv_deleted=0
    _dv_output_file=`mktemp -t ${module_name}.XXXXXXXXX`

    if [ ${_dv_deleted} -eq 1 ]
    then
        modflag=1
        myecho --pack ${module_name} "deleting old parameters ..."
        cat ${_dv_output_file}
    fi

    rm -f ${_dv_output_file}
}

#------------------------------------------------------------------------------
# create new configuration
#------------------------------------------------------------------------------
create_config ()
{
    local _cc_idx
    local _cc_imax

    myecho --pack ${module_name} "updating/creating configuration ..."

    {
        #------------------------------------------------------------------------------
        printgpl -conf "${module_name}" "2010-07-08" "jed" "Juergen Edner"
        #------------------------------------------------------------------------------

        if [ "${LPRNG_POWER_DO_DEBUG}" = "yes" -o "${LPRNG_POWER_DO_DEBUG}" = "no" ]
        then
            # debug active
            printvar "LPRNG_POWER_DO_DEBUG" "debug mode: yes or no"
            echo
        else
            # debug parameter exists but not active
            if [ -n "`grep LPRNG_POWER_DO_DEBUG ${source_conf}`" ]
            then
                echo "# LPRNG_POWER_DO_DEBUG='yes'                 # debug mode: yes or no"
                echo
            fi
        fi

        #------------------------------------------------------------------------------
        printgroup "General settings"
        #------------------------------------------------------------------------------
        printvar "START_LPRNG_POWER"               "activate configuration: yes or no"
        echo

        if variable_set "LPRNG_POWER_LOAD_CONTROL_SCRIPT"
        then
            printvar "LPRNG_POWER_LOAD_CONTROL_SCRIPT" "force load of control script"
            echo
        fi

        printvar "LPRNG_POWER_N"                   "number of power switches"

        if [ ${LPRNG_POWER_N} -eq 0 ]
        then
            _cc_imax=1
        else
            _cc_imax=${LPRNG_POWER_N}
        fi

        _cc_idx=1
        while [ ${_cc_idx} -le ${_cc_imax} ]
        do
            printvar "LPRNG_POWER_${_cc_idx}_ACTIVE"            " ${_cc_idx}. activate: yes or no"
            printvar "LPRNG_POWER_${_cc_idx}_COMMENT"           "   comment/description"
            printvar "LPRNG_POWER_${_cc_idx}_IPADDR"            "   ip address and optional port"
            printvar "LPRNG_POWER_${_cc_idx}_URL_ON"            "   url path to switch the device on"
            printvar "LPRNG_POWER_${_cc_idx}_URL_OFF"           "   url path to switch the device off"
            printvar "LPRNG_POWER_${_cc_idx}_URL_STATUS"        "   url path to get switch status"
            printvar "LPRNG_POWER_${_cc_idx}_URL_STATUS_FILTER" "   filter cmd to extract status: 1/0"
            printvar "LPRNG_POWER_${_cc_idx}_PRINTER_CAPNAME"   "   name of printer"
            printvar "LPRNG_POWER_${_cc_idx}_AUTH"              "   authentication: yes or no"
            printvar "LPRNG_POWER_${_cc_idx}_USER"              "   username"
            printvar "LPRNG_POWER_${_cc_idx}_PASS"              "   password"
            printvar "LPRNG_POWER_${_cc_idx}_TIMEOUT"           "   auto switch-off timeout in minutes"

            _cc_idx=`expr ${_cc_idx} + 1`
        done

        #------------------------------------------------------------------------------
        printend
        #------------------------------------------------------------------------------
    } > ${generate_conf}

    # set access rights
    chmod 0644 ${generate_conf}
    chown root ${generate_conf}
}

#==============================================================================
# main
#==============================================================================

pgmname=`basename $0`
module_name=`basename -s .sh ${pgmname} | cut -d- -f1`

configfile=/etc/config.d/${module_name}
installfile=/var/run/${module_name}.install

# setting defaults
source_conf=${installfile}
generate_conf=${configfile}

goflag=0
modflag=0

case "$1" in
    *-update|update )
        goflag=1
        ;;
    *-test|test )
        source_conf=${configfile}

        generate_conf=/etc/config.d/${module_name}.test
        goflag=1
        ;;
    * )
        echo
        echo "Use one of the following options:"
        echo
        echo "  ${module_name}-update.sh [update] - the file ${configfile}.import will be read, the configuration will"
        echo "                                   be checked and an updated mail configuration file will be written."
        echo
        goflag=0
        ;;
esac

if [ ${goflag} -eq 1 ]
then
    if [ -f ${source_conf} ]
    then
        # previous configuration file exists
        myecho --pack ${module_name} --info "previous configuration found ..."
        . ${source_conf}

        rename_variables
        modify_variables
        add_variables
        delete_variables

        create_config
    else
        myecho --pack ${module_name} --error "no configuration ${source_conf} found - exiting."
        myanykey --pack ${module_name}
    fi
fi

#==============================================================================
# End
#==============================================================================
exit 0
