#!/bin/sh
#-------------------------------------------------------------------------------
# /etc/config.d/lprng_power.sh - configuration generator script
#
# Copyright (c) 2010-2018  The Eisfair Team, team(at)eisfair(dot)org
#
# Creation:     2010-07-08  fm
# Last Update:  $Id: lprng_power.sh 55703 2018-06-10 18:49:02Z jed $
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#-------------------------------------------------------------------------------

# read eislib etc.
. /var/install/include/eislib
. /var/install/include/jedlib

# debug mode: true/false
#debug=true
if ${debug:-false}
then
    exec 2>/tmp/$(basename ${0})-trace$$.log
    set -x
    ask_debug=true
    export ask_debug
fi

pgmname=`basename $0`
module_name=`basename -s .sh ${pgmname} | cut -d- -f1`

### set directory and file names ###
lprng_filter=/var/install/bin/lprng-filter
lprng_power_file=/etc/config.d/${module_name}
config_file=/etc/${module_name}-access.conf
packagefile=/var/install/packages/${module_name}
generate_at_allow_file=/etc/at.allow.${module_name}
generate_filter_file=/etc/lprng-filter.${module_name}

### other parameters ###
lprng_power_version="v`grep "<version>" ${packagefile} | sed 's#<version>\(.*\)</version>#\1#'`"

### load configuration ###
. ${lprng_power_file}
chmod 600 ${lprng_power_file}

#----------------------------------------------------------------------------------------
# generate configuration
#----------------------------------------------------------------------------------------
generate_config ()
{
    if [ "${START_LPRNG_POWER}" = "yes" ]
    then
        myecho --pack ${module_name} "creating ${module_name} configuration file ..."

        {
            #----------------------------------------------------------------------------
            print_short_header "${config_file}" "${pgmname}" "${module_name}" "${lprng_power_version}"
            #----------------------------------------------------------------------------

            if [ ${LPRNG_POWER_N} -gt 0 ]
            then
                idx=1
                while [ ${idx} -le ${LPRNG_POWER_N} ]
                do
                    eval active='$LPRNG_POWER_'${idx}'_ACTIVE'

                    if [ "${active}" = "yes" ]
                    then
                        # active entry, go on...
                        eval auth='$LPRNG_POWER_'${idx}'_AUTH'
                        eval ipaddr='$LPRNG_POWER_'${idx}'_IPADDR'

                        eval url_on='$LPRNG_POWER_'${idx}'_URL_ON'
                        eval url_off='$LPRNG_POWER_'${idx}'_URL_OFF'
                        eval url_status='$LPRNG_POWER_'${idx}'_URL_STATUS'
                        eval url_status_filter='$LPRNG_POWER_'${idx}'_URL_STATUS_FILTER'

                        eval timeout='$LPRNG_POWER_'${idx}'_TIMEOUT'
                        eval capname='$LPRNG_POWER_'${idx}'_PRINTER_CAPNAME'

                        # check if protocol has been set
                        proto='http'
                        echo "${ipaddr}" | grep -q '^http'

                        if [ $? -eq 0 ]
                        then
                            proto=`echo "${ipaddr}" | cut -d: -f1`
                            ipaddr=`echo "${ipaddr}" | sed 's#^http[s]*://##'`
                        fi

                        # check if port has been set
                        port='80'
                        echo "${ipaddr}" | grep -q ':'

                        if [ $? -eq 0 ]
                        then
                            # replace colon by exclamation mark
                            port=`echo "${ipaddr}" | cut -d: -f2`
                            ipaddr=`echo "${ipaddr}" | cut -d: -f1`
                        fi

                        user=''
                        pass=''
                        if [ "${auth}" = "yes" ]
                        then
                            eval user='$LPRNG_POWER_'${idx}'_USER'
                            eval pass='$LPRNG_POWER_'${idx}'_PASS'
                        fi

                        # print header line
                        if [ ${idx} -eq 1 ]
                        then
                            echo '# queue-name!instance-number!server!port!user!password!timeout!url_on!url_off!url_status!url_status_filter'
                        fi

                        # queue-name!instance-number!protocol!server!port!user!password!timeout!url_on!url_off!url_status!url_status_filter
                        # repr1!1!http!192.168.1.49!80!admin!geheim!10!/r?b=1&r=0&s=1!/r?b=1&r=0&s=0!/xml!sed -e "/<nv0>/!d" -e "s/^.*<nv0>//" -e "s#</nv0>.*$##"
                        echo "${capname}!${idx}!${proto}!${ipaddr}!${port}!${user}!${pass}!${timeout}!${url_on}!${url_off}!${url_status}!${url_status_filter}"
                    fi

                    # create symbolic link
                    ln -sf /usr/bin/${module_name}-access /usr/bin/${module_name}-${idx}-off

                    idx=`expr ${idx} + 1`
                done
            else
                echo "# no active entries found"
            fi
        } > ${config_file}

        chmod 0644 ${config_file}
    else
        # create empty configuration
        {
            #----------------------------------------------------------------------------
            print_short_header "${config_file}" "${pgmname}" "${module_name}" "${lprng_power_version}"
            #----------------------------------------------------------------------------
            echo "disabled!"
        } > ${config_file}

        # remove symbolic links
        rm -f /usr/bin/${module_name}-*-off
    fi
}

#----------------------------------------------------------------------------------------
# create at.allow file
#----------------------------------------------------------------------------------------
generate_at_config ()
{
    if [ "${START_LPRNG_POWER}" = 'yes' ]
    then
        {
            #----------------------------------------------------------------------------------------
          # print_short_header "${generate_at_allow_file}" "${pgmname}" "${module_name}" "${lprng_power_version}"
            #----------------------------------------------------------------------------------------
            echo "lp"
        } > ${generate_at_allow_file}
    else
        rm -f ${generate_at_allow_file}
    fi

    /var/install/bin/update-at.allow ${module_name}
}

#----------------------------------------------------------------------------------------
# generate addon for lprng-filter script
#----------------------------------------------------------------------------------------
generate_filter_addon ()
{
    if [ "${START_LPRNG_POWER}" = 'yes' ]
    then
        {
            #----------------------------------------------------------------------------------------
            print_short_header "${generate_filter_file}" "${pgmname}" "${module_name}" "${lprng_power_version}"
            #----------------------------------------------------------------------------------------

            echo 'if [ -f /usr/bin/lprng_power-access ]'
            echo 'then'
            echo '    /usr/bin/lprng_power-access --queue ${queue} --on-autooff'
            echo 'fi'
        } > ${generate_filter_file}
    else
        rm -f ${generate_filter_file}
    fi
}

#----------------------------------------------------------------------------------------
# modify lprng-filter script and insert lprng_power call
#----------------------------------------------------------------------------------------
modify_lprng_filter ()
{
    grep -q "/usr/bin/lprng_power-access" ${lprng_filter}

    if [ $? -ne 0 ]
    then
        cp ${lprng_filter} ${lprng_filter}.LPRNG
        sed "s#^exit 0#\n\# ${EISDATE} inserted by lprng_power.sh script\nif [ -f /usr/bin/lprng_power-access ]\nthen\n    /usr/bin/lprng_power-access --queue \${queue} --on-autooff\nfi\n\nexit 0#" ${lprng_filter}.LPRNG > ${lprng_filter}
    fi
}

#========================================================================================
# main
#========================================================================================

force_quiet_run=false
if [ $# -gt 0 ]
then
    # read parameter(s)
    while [ $# -gt 0 ]
    do
        case $1 in
            *-quiet )
                force_quiet_run=true
                shift
                ;;
            * )
                cmd="$1"
                break
                ;;
        esac
    done
fi

case ${cmd} in
    * )
        # generate all configuration files
        myecho --pack ${module_name} ""
        myecho --pack ${module_name} "${module_name} version: ${lprng_power_version}"

        generate_config
        generate_at_config
        generate_filter_addon
        ;;
esac

#========================================================================================
# End
#========================================================================================
exit 0
